import math
import pprint
import statistics
import sys
import time
import csv
import cv2 as cv

# Fix path when importing files from different folder
sys.path.append("./")
from feature import (
    findKeyPoints,
    findMatches,
    filterMatches,
    distanceBetweenMatches,
)
from utils import rescaleFrame


def convertDotToComma(value):
    # Convert value to comma seperated string
    # Later analyzing is done in google sheets which does not like dot seperation between integers and decimals
    return str(format(value, ".2f")).replace(".", ",")


def writeDataToFile(data):
    with open("./Testing/Results/test2.csv", "w", newline="") as file:
        # Open CSV writer with delimiter (value seperator) as ;
        writer = csv.writer(file, delimiter=";")

        writer.writerow(
            [
                "Bilde indeks",
                "Avstand estimert",
                "Avstand referanse",
                "Differanse fra referanse",
                "Relativ feil",
                "Nøyaktighet",
            ]
        )

        for key, value in data.items():

            writer.writerow(
                [
                    key,
                    convertDotToComma(value["Avstand estimert"]),
                    convertDotToComma(value["Avstand referanse"]),
                    convertDotToComma(value["Differanse fra referanse"]),
                    convertDotToComma(value["Relativ feil"]),
                    convertDotToComma(value["Nøyaktighet"]),
                ]
            )


# Constants
VIDEO_NAME = "sim_realtime.mp4"
VIDEO_FPS = 30
VIDEO_START_INDEX = 0
NUM_OF_FRAMES_TO_SKIP = 5
KEYPOINTS_METHOD = "ORB"
REFERENCE_SPEED = 0.2  # In meters per second

# Find total number of frames in video
cap = cv.VideoCapture("./Resources/" + VIDEO_NAME)
NUM_OF_FRAMES_IN_VIDEO = int(cap.get(cv.CAP_PROP_FRAME_COUNT))

# 3cm distance between squares in net
DISTANCE_BETWEEN_SQUARES = 0.03
NUM_OF_SQUARES_IN_FRAME = 15.5
VIDEO_HEIGHT = 720
DISTANCE_PER_PX = (DISTANCE_BETWEEN_SQUARES * NUM_OF_SQUARES_IN_FRAME) / VIDEO_HEIGHT
REFERENCE_DISTANCE = (1 / 30) * REFERENCE_SPEED

# Open capture of video at the set start index
cap = cv.VideoCapture("./Resources/" + VIDEO_NAME)

# Read the first frame
ret, frame1 = cap.read()
frame1 = frame1[0:100, 100:400]

# Find keypoints and descriptors for the first frame
kp1, des1 = findKeyPoints(frame1, KEYPOINTS_METHOD)

distances = []
data = {}

for i in range(0, 40):
    # Read the N'th frame from the capture
    ret, frameN = cap.read()

    if not ret:
        print("No frame found for index ", i)

    frameN = frameN[0:100, 100:400]

    # Find keypoints and descriptors for the N'th frame
    kpN, desN = findKeyPoints(frameN, KEYPOINTS_METHOD)

    # Find matches between frames
    matches = findMatches(frame1, frameN, kp1, kpN, des1, desN)

    # Calculate the estimated distance based on the matches found
    distance = distanceBetweenMatches(matches, kp1, kpN)

    # Convert distance in px to distance in meters
    distanceInMeters = distance * DISTANCE_PER_PX

    # Save distance for later analyzing
    distances.append(distanceInMeters)

for i in range(0, len(distances)):
    d = distances[i] * -1
    N = i + 1
    referenceDistance = N * REFERENCE_DISTANCE

    # Calculate relative error with reference value and estimated
    relError = (abs(d - referenceDistance)) / abs(referenceDistance)

    # Calculate accuracy with reference value and average
    accuracy = (
        referenceDistance - (max(referenceDistance, d) - min(referenceDistance, d))
    ) / referenceDistance

    # If the calculated accuracy is outside of the 0-100% range of the reference value
    # the accuracy must be 0
    if accuracy < 0:
        accuracy = 0

    differenceFromReference = max(d, referenceDistance) - min(d, referenceDistance)

    distance = round(d, 2)
    accuracy = round(accuracy * 100, 2)
    differenceFromReference = round(differenceFromReference, 2)
    referenceDistance = round(referenceDistance, 2)
    relError = round(relError * 100, 2)

    data[str(i)] = {
        "Nøyaktighet": accuracy,
        "Relativ feil": relError,
        "Avstand referanse": referenceDistance,
        "Avstand estimert": distance,
        "Differanse fra referanse": differenceFromReference,
    }

writeDataToFile(data)