import numpy as np
import cv2 as cv
import sys
import time

import multiprocessing as mp
import threading
from os import system, name

# Fix path when importing files from different folder
sys.path.append("./")
from utils import rescaleFrame
from image import cannyEdgeDetecion, findKeyPoints, findMatches, distanceBetweenMatches
from measurements import calculateSpeed, calculatePosition


Thread = threading.Thread

# CONSTANTS
NUM_OF_FRAMES_BETWEEN_ESTIMATION = 1
KEYPOINTS_METHOD = "ORB"


def clear():
    # Clear console log in windows
    if name == "nt":
        _ = system("cls")


def printTime(times):
    # Clear previous outputs
    clear()

    for t in times:
        print(t[0], convertTime(t[1]))


def convertTime(time):
    return str(round(time * 1000, 2)) + "ms"


def updateTime(timePrev, timeBefore, timeAfter):
    timeNew = abs(timeAfter - timeBefore)
    return (timePrev + timeNew) / 2


cap = cv.VideoCapture("./Resources/sim2_realtime.mp4")
cap.set(cv.CAP_PROP_POS_FRAMES, 0)


_, framePrev = cap.read()
_, frame = None, None
kpPrev, desPrev = findKeyPoints(framePrev, KEYPOINTS_METHOD)


frameIndex = 0
speedPrev = 0
speed = 0
x, y = 0, 0
kp, des = None, None


matchesBetweenFrames = []

kpTime = 0
matchTime = 0
drawingTime = 0
totalPerFrame = 0

running = False


def parallellTask():
    global kp, des
    global kpPrev, desPrev
    global framePrev
    global frame
    global kpTime
    global matchTime
    global speedPrev
    global x, y
    global speed
    global running
    global kpTime, matchTime

    # framesList = []

    while running:
        # print("Thread 1 running!")
        timeBeforeKp = time.time()
        kp, des = findKeyPoints(frame, KEYPOINTS_METHOD)
        timeAfterKp = time.time()

        timeBeforeMatching = time.time()
        matchesBetweenFrames = findMatches(frame, framePrev, kp, kpPrev, des, desPrev)
        timeAfterMatching = time.time()

        distance = distanceBetweenMatches(matchesBetweenFrames, kp, kpPrev)

        speed, speedPrev = calculateSpeed(
            distance, speedPrev, NUM_OF_FRAMES_BETWEEN_ESTIMATION
        )

        x, y = calculatePosition(x, y, speed, NUM_OF_FRAMES_BETWEEN_ESTIMATION)

        kpTime = updateTime(kpTime, timeBeforeKp, timeAfterKp)
        matchTime = updateTime(matchTime, timeBeforeMatching, timeAfterMatching)
        # print(kpTime)
        # if kpTime > 0.300:
        #     framesList.append(frameIndex)

        kpPrev, desPrev = kp, des
        kp, des = None, None

    # print(framesList)


def frameTask():
    global frame
    global framePrev
    global kpTime
    global matchTime
    global frameIndex
    global drawingTime
    global running

    while True:
        # if frameIndex == 150:
        #     running = False
        #     break

        # print("Thread 2 running!")

        _, frame = cap.read()
        # print(kp, des)

        matchesFrame = frame

        if not running:
            # print("new thread")
            t2 = Thread(None, parallellTask, None, [])
            running = True
            t2.start()

        # else:
        # Parallell testing
        frameCopy = frame.copy()
        framePrevCopy = framePrev.copy()

        framePrev = frame
        # kpPrev, desPrev = kp, des

        # ---------------------------------------
        # -------- DRAWING AND RENDERING --------
        # ---------------------------------------

        # Add text to show which frame is which
        cv.putText(
            frameCopy,
            "Current frame",
            (50, 50),
            cv.FONT_HERSHEY_TRIPLEX,
            1.0,
            (255, 255, 255),
            2,
        )
        cv.putText(
            frameCopy,
            "Frame index: " + str(frameIndex),
            (50, 90),
            cv.FONT_HERSHEY_TRIPLEX,
            1.0,
            (255, 255, 255),
            2,
        )

        cv.putText(
            framePrevCopy,
            "Previous frame",
            (50, 50),
            cv.FONT_HERSHEY_TRIPLEX,
            1.0,
            (255, 255, 255),
            2,
        )

        # Add speed and positon text
        cv.putText(
            frameCopy,
            "Speed: {0:.2g}".format(speed) + "m/s",
            (210, 180),
            cv.FONT_HERSHEY_TRIPLEX,
            1.0,
            (0, 255, 255),
            2,
        )

        cv.putText(
            frameCopy,
            "Position: X{0:.2g}".format(x) + "m, Y{0:.2g}".format(y) + "m",
            (210, 225),
            cv.FONT_HERSHEY_TRIPLEX,
            1.0,
            (0, 255, 255),
            2,
        )

        timeBeforeDrawing = time.time()
        # Need to draw only good matches, so create a mask
        matchesMask = [[0, 0] for i in range(len(matchesBetweenFrames))]

        # ratio test as per Lowe's paper
        for i, (m, n) in enumerate(matchesBetweenFrames):
            if m.distance < 0.7 * n.distance:
                matchesMask[i] = [1, 0]

        draw_params = dict(
            matchColor=(0, 255, 0),
            singlePointColor=(0, 0, 255),
            matchesMask=matchesMask,
            flags=0,
        )

        # matchesFrame = cv.drawMatchesKnn(
        #     framePrevCopy,
        #     kpPrev,
        #     frameCopy,
        #     kp,
        #     matchesBetweenFrames,
        #     None,
        #     **draw_params
        # )
        timeAfterDrawing = time.time()

        # kpTime = updateTime(kpTime, timeBeforeKp, timeAfterKp)
        # matchTime = updateTime(matchTime, timeBeforeMatching, timeAfterMatching)
        drawingTime = updateTime(drawingTime, timeBeforeDrawing, timeAfterDrawing)
        totalPerFrame = kpTime + matchTime + drawingTime

        times = [
            ["Keypoints time:", kpTime],
            ["Matching time:", matchTime],
            ["Drawing time", drawingTime],
            ["Total per frame", totalPerFrame],
        ]

        # printTime(times)

        # else:
        #     distanceBetweenEstimation += distance

        frameIndex += 1

        cv.imshow("matches", rescaleFrame(frameCopy, 0.4))
        # cv.imshow("frame", rescaleFrame(frame, 0.4))

        k = cv.waitKey(30) & 0xFF

        # Show the Output
        if k == 27:
            running = False
            break


t = Thread(None, frameTask, None, ())
t.start()


for thread in threading.enumerate():
    print(thread.name)

# if t.is_alive():
#     t.threading.Event().set()