import numpy as np
import cv2 as cv
import sys

# Fix path when importing files from different folder
sys.path.append("./")
# from utils import rescaleFrame
# from image import cannyEdgeDetecion

cap = cv.VideoCapture("./Resources/sim_realtime.mkv")

position = [0, 0]
previous_position = [0, 0]

round_position = [0, 0]


def calc_pos(x, y, size=1, last_pos=[0, 0]):

    if x == last_pos[0] or y == last_pos[1]:
        return
    position[0] = position[0] + ((-x + last_pos[0]) * size // 200)
    position[1] = position[1] + ((-y + last_pos[1]) * size // 200)
    # print((f'Position: X({position[0]}cm), Y({position[1]}cm)'))


def calc_rounds(x, y):
    round_position[0] = round_position[0] + x
    round_position[1] = round_position[1] + y


# take first frame of the video
print(cap.get(3))
print(cap.get(4))
cap.set(cv.CAP_PROP_POS_FRAMES, 100)
ret, frame = cap.read()


# setup initial location of window
x, y, w, h = 700, 700, 200, 200  # simply hardcoded the values
track_window = (x, y, w, h)
# set up the ROI for tracking
roi = frame[y : y + h, x : x + w]
hsv_roi = cv.cvtColor(roi, cv.COLOR_BGR2HSV)
mask = cv.inRange(hsv_roi, np.array((0.0, 60.0, 32.0)), np.array((180.0, 255.0, 255.0)))
roi_hist = cv.calcHist([hsv_roi], [0], mask, [180], [0, 180])
cv.normalize(roi_hist, roi_hist, 0, 255, cv.NORM_MINMAX)
# Setup the termination criteria, either 10 iteration or move by atleast 1 pt
term_crit = (cv.TERM_CRITERIA_EPS | cv.TERM_CRITERIA_COUNT, 10, 1)
while 1:
    ret, frame = cap.read()
    if ret == True:
        hsv = cv.cvtColor(frame, cv.COLOR_BGR2HSV)
        # cv.imshow('HSV',hsv)
        dst = cv.calcBackProject([hsv], [0], roi_hist, [0, 180], 1)
        # apply meanshift to get the new location

        ret, track_window = cv.meanShift(dst, track_window, term_crit)
        # Draw it on image
        # Separate placement if it hits top or bottom.
        previous_position[0] = x
        previous_position[1] = y
        if y == 0 or x == 0 or y == 1080 or x == 1920:
            if y == 0:
                # 5cm*11 notes = 55cm
                calc_rounds(0, 55)
            if y == 1080:
                # 5cm*11 notes = 55cm
                calc_rounds(0, -55)
            if x == 0:
                # 5cm*20 notes = 100cm
                calc_rounds(100, 0)
            if y == 1920:
                # 5cm*20 notes = 100cm
                calc_rounds(-100, 0)
            x, y, w, h = 700, 700, 200, 200
            track_window = (x, y, w, h)
            previous_position[0] = x
            previous_position[1] = y

        else:
            x, y, w, h = track_window

        calc_pos(x, y, 16, previous_position)

        cv.putText(
            frame,
            f"Position: X({position[0]}cm), Y({position[1]}cm)",
            (225, 225),
            cv.FONT_HERSHEY_TRIPLEX,
            1.0,
            (0, 255, 255),
            2,
        )
        cv.putText(
            frame,
            f"Round Position: X({round_position[0]}cm), Y({round_position[1]}cm)",
            (225, 255),
            cv.FONT_HERSHEY_TRIPLEX,
            1.0,
            (0, 255, 255),
            2,
        )

        img2 = cv.rectangle(frame, (x, y), (x + w, y + h), 255, 2)

        cv.imshow("img2", img2)
        k = cv.waitKey(1) & 0xFF
        if k == 27:
            break
    else:
        break