import numpy as np
import cv2 as cv
import sys
import time
from utils import rescaleFrame
from image import (
    cannyEdgeDetecion,
    findKeyPoints,
    findMatches,
    distanceBetweenMatches,
    filterMatches,
    flattenHistogram,
)
from measurements import calculateSpeed, calculatePosition

VIDEO_NAME = "./Resources/sim_realtime.mp4"
KEYPOINTS_METHOD = "ORB"
FRAME_START_INDEX = 0
FRAME_TO_SKIP = 5

cap = cv.VideoCapture(VIDEO_NAME)
cap.set(cv.CAP_PROP_POS_FRAMES, FRAME_START_INDEX)


def findFeatures(frame):
    # frame = flattenHistogram(frame)
    # frame = cannyEdgeDetecion(frame)
    # Invert image
    # frame = cv.bitwise_not(frame)

    # Current frame keypoints and descriptors
    kp, des = findKeyPoints(frame, KEYPOINTS_METHOD, 20)

    return kp, des, frame


# Save previous frame
_, framePrev = cap.read()
# framePrev = cv.imread("./Resources/book.jpg")

# Skip n-number of frames for next frame in estimation
cap.set(cv.CAP_PROP_POS_FRAMES, FRAME_START_INDEX + FRAME_TO_SKIP)
_, frame = cap.read()
# frame = cv.imread("./Resources/books.jpg")

kpPrev, desPrev, framePrevNew = findFeatures(framePrev)
kp, des, frameNew = findFeatures(frame)

matchesBetweenFrames = findMatches(frame, framePrev, kp, kpPrev, des, desPrev)

matchesDownwards, matchesUpwards = filterMatches(matchesBetweenFrames, kp, kpPrev)

for i in range(0, len(matchesBetweenFrames)):
    m = matchesBetweenFrames[i]
    index1 = m[0].trainIdx
    index2 = m[0].queryIdx

    x1, y1 = kp[index1].pt[0], kp[index1].pt[1]
    x2, y2 = kpPrev[index2].pt[0], kpPrev[index2].pt[1]

    # print("index", index1, index2)
    # print("kp1: ", kp[index1].pt, ", kp2: ", kpPrev[index2].pt)

    cv.putText(
        frame,
        "kp1, " + str(i),
        (int(x1), int(y1)),
        cv.FONT_HERSHEY_TRIPLEX,
        1.0,
        (0, 255, 255),
        2,
    )

    cv.putText(
        framePrev,
        "kp2, " + str(i),
        (int(x2), int(y2)),
        cv.FONT_HERSHEY_TRIPLEX,
        1.0,
        (0, 255, 255),
        2,
    )

draw_params = dict(
    # matchColor=(0, 255, 0),
    # singlePointColor=(0, 0, 255),
    # matchesMask=matchesMaskFiltered,
    # flags=4,
)

draw_params2 = dict(
    # matchColor=(0, 0, 255),
    # singlePointColor=(255, 0, 0),
    # matchesMask=matchesMaskFiltered,
    flags=2,
)

matchesFrame = cv.drawMatchesKnn(
    framePrev, kpPrev, frame, kp, matchesDownwards, None, **draw_params2
)

matchesFrame2 = cv.drawMatchesKnn(
    framePrev, kpPrev, frame, kp, matchesUpwards, None, **draw_params2
)

matchesFrame3 = cv.drawMatchesKnn(
    framePrev, kpPrev, frame, kp, matchesBetweenFrames, None, **draw_params2
)

print("kp1 length: ", len(kp))
print("kp2 length: ", len(kpPrev))
print("matches length: ", len(matchesBetweenFrames))

SCALE = 0.6

cv.imshow("matches downwards", rescaleFrame(matchesFrame, SCALE))
cv.imshow("matches upwards", rescaleFrame(matchesFrame2, SCALE))
cv.imshow("matches", rescaleFrame(matchesFrame3, SCALE))

k = cv.waitKey(0)